#
# users.py - Handles logins and other user interaction
#
# Copyright 2013, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>
#
# Liberally borrows from the sample app in Flask-FAS maintained by Ian Weller

from flask import Blueprint, render_template, redirect, flash, url_for, request, g, abort
from functools import wraps

from flask.ext.wtf import Form, TextField, PasswordField, HiddenField

from blockerbugs import app, fas

users = Blueprint('users', __name__)


class LoginForm(Form):
    username = TextField(u'Username')
    password = PasswordField(u'Password')
    next_url = HiddenField()


@users.route('/login', methods=['GET', 'POST'])
def login():
    login_form = LoginForm()
    if 'next' in request.args:
        next_url = request.args['next']
        login_form.next_url.data = next_url
    else:
        next_url = url_for('main.index')

    # If user is already logged in, return them to where they were last
    if g.fas_user:
        return redirect(next_url)
    return fas.login(return_url=next_url)


@users.route('/logout')
def logout():
    if g.fas_user:
        app.logger.info('Logging out user %s' % g.fas_user.username)
        fas.logout()
    else:
        app.logger.debug('No FAS user to logout')
    return redirect(url_for('main.index'))

# This is a decorator we can use with any HTTP method (except login, obviously)
# to require a login. In this application it is only used with claplusone and
# secret. If the user is not logged in, it will redirect them to the login form.
# http://flask.pocoo.org/docs/patterns/viewdecorators/#login-required-decorator
def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if g.fas_user is None:
            return redirect(url_for('users.login', next=request.url))
        return f(*args, **kwargs)
    return decorated_function


def admin_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if g.fas_user is None:
            return redirect(url_for('users.login', next=request.url))
        else:
            for group in g.fas_user.approved_memberships:
                if group.name == app.config['FAS_ADMIN_GROUP']:
                    return f(*args, **kwargs)
            app.logger.info('Failed admin access to %s by %s' % (request.url, g.fas_user.username))
            abort(401)
    return decorated_function
