#
# bug.py - ORM class for bug information stored in the database
#
# Copyright 2012, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>


from blockerbugs import db
import datetime


class Bug(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    bugid = db.Column(db.Integer, unique=False)
    url = db.Column(db.String(1024), unique=False)
    summary = db.Column(db.String(2048), unique=False)
    status = db.Column(db.String(80), unique=False)
    component = db.Column(db.String(80), unique=False)
    active = db.Column(db.Boolean)
    last_bug_sync = db.Column(db.DateTime)
    proposed_blocker = db.Column(db.Boolean)
    proposed_fe = db.Column(db.Boolean)
    rejected_blocker = db.Column(db.Boolean)
    rejected_fe = db.Column(db.Boolean)
    accepted_blocker = db.Column(db.Boolean)
    accepted_fe = db.Column(db.Boolean)
    milestone_id = db.Column(db.Integer, db.ForeignKey('milestone.id'))
    milestone = db.relationship('Milestone',
                                backref=db.backref('bugs', lazy='dynamic'))
                                #backref='bugs')

    def __init__(self, bugid, url, summary, status, component, milestone,
                 active, last_bug_sync=datetime.datetime.now()):
        self.bugid = bugid
        self.url = url
        self.summary = summary
        self.status = status
        self.component = component
        self.active = active
        self.milestone = milestone
        self.proposed_blocker = False
        self.accepted_blocker = False
        self.rejected_blocker = False
        self.proposed_fe = False
        self.accepted_fe = False
        self.rejected_fe = False
        self.last_bug_sync = last_bug_sync

    def __repr__(self):
        return '<bug %d: %s>' % (self.bugid, self.summary)

    def update(self, buginfo, tracker_type, release):
        self.url = buginfo['url']
        self.summary = buginfo['summary']
        self.status = buginfo['status']
        self.component = buginfo['component']
        self.last_bug_sync = buginfo['last_change_time']
        self.active = buginfo['active']
        if tracker_type == 'Blocker':
            self.proposed_blocker = buginfo['proposed']
            self.accepted_blocker = buginfo['accepted']
            self.rejected_blocker = buginfo['rejected']
        elif tracker_type == 'FreezeException':
            self.proposed_fe = buginfo['proposed']
            self.accepted_fe = buginfo['accepted']
            self.rejected_fe = buginfo['rejected']
        if release.id != self.milestone.id:
            raise ValueError('Bugs with the same id and different releases should be different rows!')

    @classmethod
    def from_data(cls, buginfo, release, tracker_type):
        newbug = Bug(buginfo['bug_id'], '', '', '','', release, buginfo['active'])
        newbug.update(buginfo, tracker_type, release)
        return newbug

