#
# milestone.py - ORM class for release milestone info stored in the database
#
# Copyright 2012, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>


from blockerbugs import db
from blockerbugs.models.criterion import Criterion


class Milestone(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    release_id = db.Column(db.Integer, db.ForeignKey('release.id'))
    release = db.relationship('Release', backref=db.backref('milestones', lazy='dynamic'))
    version = db.Column(db.String(80))
    name = db.Column(db.String(80), unique=True)
    blocker_tracker = db.Column(db.Integer, unique=True)
    fe_tracker = db.Column(db.Integer, unique=True)
    active = db.Column(db.Boolean)
    last_bug_sync = db.Column(db.DateTime)
    current = db.Column(db.Boolean)
    criteria = db.relationship('Criterion', backref='milestone', lazy='dynamic')
    succeeds_id = db.Column(db.Integer, db.ForeignKey('milestone.id'), nullable=True)
    succeeds = db.relationship('Milestone',
                               backref=db.backref('succeeded_by',
                                                  remote_side=[id],
                                                  uselist=False),
                               lazy='dynamic')

    def __init__(self, release, version, blocker_tracker, fe_tracker,
                 name, active=False, current=False, succeeds=None):
        self.release = release
        self.version = version
        self.blocker_tracker = blocker_tracker
        self.fe_tracker = fe_tracker
        self.name = name
        self.active = active
        self.last_bug_sync = None
        self.current = current
        if succeeds:
            self.succeeds = succeeds

    def __repr__(self):
        return "<milestone: %d-%s>" % (self.release.number, self.version)
