#
# spin.py - database model for spins
#
# Copyright 2012, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>
from blockerbugs import db
from datetime import datetime

from blockerbugs import db
from blockerbugs.models import DeclEnum
import datetime

used_updates = db.Table('used_updates',
        db.Column('update_id', db.Integer, db.ForeignKey('update.id')),
        db.Column('spin_id', db.Integer, db.ForeignKey('spin.id'))
)


class SpinState(DeclEnum):
    requested = 'requested', 'requested'
    built = 'built', 'built'
    obsolete = 'obsolete', 'obsolete'


class Spin(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(160), unique=False)
    state = db.Column(SpinState.db_type())
    url = db.Column(db.String(2048), nullable=True)
    date_requested = db.Column(db.DateTime, unique=False)
    date_created = db.Column(db.DateTime, unique=False)
    spin_type = db.Column(db.String(80), unique=False)
    updates = db.relationship('Update', secondary=used_updates,
                              backref=db.backref('spins', lazy='dynamic'))
    succeeds_id = db.Column(db.Integer, db.ForeignKey('spin.id'), nullable=True)
    succeeds = db.relationship('Spin', backref=db.backref('succeeded_by', remote_side=id, uselist=False))
                                #lazy='dynamic')
    milestone_id = db.Column(db.Integer, db.ForeignKey('milestone.id'))
    milestone = db.relationship('Milestone',
                                backref=db.backref('spins', lazy='dynamic'))

    def __init__(self, name, spin_type, updates, milestone,
                 succeeds=None, date_requested=datetime.datetime.utcnow(),
                 date_created=None, state=SpinState.requested,
                 url=''):
        self.name = name
        self.spin_type = spin_type
        self.updates = updates
        self.milestone = milestone
        if succeeds:
            self.succeeds = succeeds
        self.date_requested = date_requested
        self.date_created = None
        self.state = state
        self.url = url

    def __repr__(self):
        return "<spin: %s>" % self.name
